//-----------------------------------------------------------------------------
//
// c2d
//
// Syntax: G=c2d(A,B,T)
//
// This routine converts a continuous time system given by:
//      .
//      x = Ax + Bu
//
// to the discrete time state space system:
//
//      x[n+1] = phi * x[n] + gamma * u[n]
//
// It assumes that there is a zero-hold on the inputs and that
// it has a sampling time of T.
//
// The results are returned in a list:
//
//      G.phi = phi
//      G.gamma = gamma
//
// Ref:  Frankin, G. F., Powell, J. D., Workman, M. L., "Digital Control of
//       Dynamic Systems," Addison-Wesley, Reading Mass., 1990.
//
// Note: This zero order hold conversion was just taken from the ref.
//       as a special case of the first order hold with triangle
//       approximation.
//
// Copyright (C), by Jeffrey B. Layton, 1994
// Version JBL 940918
//-----------------------------------------------------------------------------

require abcdchk expm

c2d = function(A,B,T)
{

   if (nargs < 3) {
       error("C2D: Wrong number of Input arguments.");
   }

   // Check A,B to ensure that it is correct
   msg="";
   msg=abcdchk(A,B);
   if (msg != "") {
       estr="C2D: "+msg;
       error(estr);
   }

   // Form FT matrix and take matrix exponential
   FT=[[A,B]*T;
       zeros(B.nc,A.nc+B.nc)];
   E=expm(FT);

   // Find Phi and Gamma partitions of E
   phi=E[1:A.nc;1:A.nc];
   gamma=E[1:A.nc;(A.nc+1):(A.nc+B.nc)];

   return << phi=phi; gamma=gamma >>;
};

