//-------------------------------------------------------------------//
//  fsolve
//
//  Syntax: R = fsolve(FUN, X0, tol, method)
//
//  Description:
//
//    Find root(s) R of function FUN.
//    fsolve starts at the value X0 and tries to solve the 
//    root of functions described in FUN.  
//    tol is an optional parameter for specifying the minimum simplex 
//    size (default 1e-5).
//    method=0 use mdsmax() (default)
//          =1 use nmsmax()
//
//  Note:
//    This rfile converts the FUN(x)=0 problem into a 
//    maximization of -(FUN(x).^2) problem.    
//
//  Example 1: 
//  find solution of x-1=0, start from x0=0
// 
//  > f = function(x) { return x-1; };
//  > r = fsolve(f,0)
//   r =
//          1  
//
//  Example 2:
//  find a non-trivial solution of tan(x)=x.
//  > f = function(x) { return tan(x)-x; };
//  > r = fsolve(f,0.5)
//   r =
//          0  
//  // trivial solution.   let's try x0=2
//  > format(12,6);
//  > r = fsolve(f,2)
//   r =
//       4.49341  
//
//  Example 3:
//  find the solutions of the following set of equations:
//       x^2 + y^2 = 10
//       5*x + y   = 8
//
//  > format(12,6);
//  > f=function(x) { return [x[1]^2+x[2]^2-10; 5*x[1]+x[2]-8];};
//  > r1=fsolve(f,[0;0])
//   r1 =
//       2.07693
//      -2.38461
//  // get one solution.
//  // let's try using nmsmax to solve it  
//  > r2=fsolve(f,[0;0],,1)
//   r2 =
//      0.999996  
//      2.99999  
//  // how lucky, this is the other solution
//
//  See Also: mdsmax, nmsmax
//
//  Tzong-Shuoh Yang (yang@isec.com)  6/25/96 
//
//-------------------------------------------------------------------//
require mdsmax nmsmax 
static  (funct,internal)

fsolve = function( fun, x0, tol, method)
{
    // copy the function into an internal variable
    funct  = fun;
    // some parameters used in mdsmax/nmsmax
    // change them for better performance
    // see mdsmax/nmsmax for detailed description
    if(exist(tol)) {
      stopit = [tol,1e5,inf(),0,0];
    else
      stopit = [1e-5,1e5,inf(),0,0];
    }
    meth = 0;
    if (exist(method)) {
       if (method==0) {
          meth = 0;
       else
          meth = 1;
       }
    }
    // you can use either mdsmax or nmsmax here
    if (meth) {
       result = nmsmax(internal, x0, stopit);
    else
       result = mdsmax(internal, x0, stopit);
    }
    // check the solution 
    if (-result.fmax > 1e-2) {
       printf("fsolve: please check the feasibility of the solution\n");
    }  
    // return the solution x
    return result.x;
};

// this function simply converts the fun(x)=0 problem into a
// maximization problem. The maximum is zero at the root.
internal = function(x)
{
    return -sum((funct(x).^2)[:]);
};
