//-------------------------------------------------------------------//
//  maximize_eq, maximize_le, maximize_gen
//
//  Syntax: X = maximize_eq (F, H, X0, tol, method)
//          X = maximize_le (F, G, X0, tol, method, strategy)
//          X = maximize_gen(F, G, H, X0, tol, method, strategy) 
//
//  Description:
//
//    find solution X which maximizes function F(X) subject to 
//    equality and/or inequality constraint function(s).
//  
//    maximize_eq can be used to solve problem:
//
//                 maximize   F(X)
//                 subject to H(X) = 0
//
//    maximize_le can be used to solve problem:
//
//                 maximize   F(X)
//                 subject to G(X) <= 0
//
//    maximize_gen can be used to solve problem:
//
//                 maximize   F(X)
//                 subject to G(X) <= 0
//                            H(X)  = 0
//
//  Arguments:
//
//    F(X)     user defined objective function to be maximized
//             note: to minimize a function F(X) is equivalent
//                   to maximize -F(X)
//    H(X)     user defined equality   constraint function(s)
//    G(X)     user defined inequality constraint function(s)
//             for best result, user should supply a set of
//             well-scaled G & H functions 
//             (e.g. dimensionless, normalized constraints)
//    X0       initial value (can be a scalar, a vector, or a matrix)   
//             the output X will be the same form as X0
//    tol      an optional parameter for specifying the minimum simplex 
//             size for unconstrained maximization (default 1e-5).
//    method   an optional parameter for unconstrained maximizer
//             = 0 use mdsmax() (default)
//             = 1 use nmsmax()
//    strategy an optional parameter for selecting a sequential 
//             unconstrained maximization technique (SUMT)
//             = 0 exterior penalty function method (default)
//             = 1 interior penalty function method (not available yet)
//             = 2 extended interior penalty function method (not available yet)
//             = 3 augmented Lagrange multiplier method (not available yet)
//
//  References:
//    1. Fiacco, A. V. & McCormick, G. P., Nonlinear Programming:
//           Sequential Unconstrained Minimization Techniques,
//           John Wiley and Sons, 1968.
//    2. Fox, R. L., Optimization Methods for Engineering Design,
//           Addison-Wesley, 1971.
//    3. Gillett, B. E., Introduction to Operations Research,
//           McGraw-Hill, 1976.
//    4. Himmelblau, D. M., Applied Nonlinear Programming, 
//           McGraw-Hill, 1972.
//    5. Luenberger, D. G., Introduction to Linear and Nonlinear
//           Programming, 2nd ed., Addison-Wesley, 1984.
//    6. Vanderplaats, G. N., Numerical Optimization Techniques for
//           Engineering Design: with Applications, McGraw-Hill, 1984.
//   
//  Keywords: Optimization, Linear/Nonlinear Programming, Operations Research 
//
//  Example 1: 
//  minimize   f(x) = (x1-1)^2 + (x1-x2)^2 + (x2-x3)^4
//  subject to h(x) = x1*(1+x2^2) + x3^4 - 3 = 0
// 
//    f = function(x)
//    {
//        v = (x[1]-1)^2 + (x[1]-x[2])^2 + (x[2]-x[3])^4;
//        return -v; // minimize f = maximize -f
//    };
//  
//    h = function(x)
//    {
//        v = x[1]*(1+x[2]^2) + x[3]^4 - 3;
//        return v;
//    };
//
//    x=maximize_eq(f,h,[2,1.4,1.5])
//    printf("minimum f(x) = %.1g\n",-f(x));
//
//  result:
//
//    x =
//            1          1      0.999  
//    minimum f(x) = 3e-07
//    > 
//
//
//  Example 2:
//  minimize   f(x)  = (x1-x2)^2 + (x2+x3-2)^2+(x4-1)^2+(x5-1)^2
//  subject to h1(x) = x1 + 3*x2 - 4 = 0
//             h2(x) = x3 + x4 -2*x5 = 0
//             h3(x) = x2 - x5 = 0
//
//    f = function(x)
//    {
//        v = (x[1]-x[2])^2 + (x[2]+x[3]-2)^2 + (x[4]-1)^2 + (x[5]-1)^2;
//        return -v; // minimize f = maximize -f
//    };
//
//    h = function(x)
//    {
//        h1 = x[1] + 3*x[2] - 4;
//        h2 = x[3] + x[4] - 2*x[5];
//        h3 = x[2] - x[5];
//        return [h1, h2, h3];
//    };
//
//    x=maximize_eq(f,h,[0,0,0,0,0])
//    printf("minimum f(x) = %.1g\n",-f(x));
//
//  Result:
//    x =
//            1          1          1          1          1  
//    minimum f(x) = 8e-09
//    > 
//
//  Example 3: from Gillett p.95, a linear programming problem
//             (simplex method is much faster for LP problems)
//  maximize   f(x)  = 6*x1 + 4*x2
//  subject to 2*x1 + 3*x2 <= 30
//             3*x1 + 2*x2 <= 24
//             x1 + x2 >= 3
//             x1, x2 >= 0 
//
//    f = function(x)
//    {
//        v = 6*x[1] + 4*x[2];
//        return v;
//    };
//
//    g = function(x)
//    {
//        g1 = 2*x[1] + 3*x[2] - 30;
//        g2 = 3*x[1] + 2*x[1] - 24;
//        g3 = 3 - x[1] - x[2];
//        g4 = -x[1]; // x1 >= 0
//        g5 = -x[2]; // x2 >= 0
//        return [g1, g2, g3, g4, g5];
//    };
//
//    x=maximize_le(f,g,[0,0],1e-6)
//    printf("max. f(x) = %.3g\n", f(x));
//
//  Result:
//            8  -4.81e-06  
//    max. f(x) = 48
//
//  Example 4: (Vanderplaats p.178)
//  minimize   f(x)  = x1^3 + x1*x2 + x2^2
//  subject to x1^2 + x2^2 = 10
//             x1, x2 >= 0 
//
//    f = function(x)
//    {
//       return -(x[1]^3+x[1]*x[2]+x[2]^2);
//    };
//    
//    h = function(x)
//    {
//       return (x[1]^2 + x[2]^2 - 10);
//    };
//
//    g = function(x)
//    {
//       return [-x[1], -x[2]];
//    };
//
//    x=maximize_gen(f,g,h,[0,0],1e-6)
//    printf("min. f(x) = %.3g\n", -f(x));
//
//  Result:
//  -9.93e-05       3.16  
//    min. f(x) = 10
//
//  See Also: mdsmax, nmsmax, fsolve
//
//  Tzong-Shuoh Yang (yang@isec.com)  6/27/96 
//
//-------------------------------------------------------------------//
require mdsmax nmsmax 
static  (funF, funG, funH, internal1, internal2, internal3)
static  (penalty, gamma)
static  (exterior1, exterior2, exterior3)
static  (interior2, interior3, alm2, alm3)
static  (einterior2, einterior3)

maximize_eq = function( F, H, x0, tol, method)
{
    global (mdsmax, nmsmax)
    // copy the function into internal variable
    funF  = F;
    funH  = H;
    // some parameters used in mdsmax/nmsmax
    if(exist(tol)) {
      stopit = [tol,1e5,inf(),0,0];
    else
      stopit = [1e-5,1e5,inf(),0,0];
    }
    // you can use either mdsmax or nmsmax here
    meth = mdsmax;
    if (exist(method)) {
       if (method==0) {
          meth = mdsmax;
       else
          meth = nmsmax;
       }
    }
    // strategy
    internal1 = exterior1;
    penalty   = 1;  
    gamma     = 3;

    result = meth(internal1, x0, stopit);
    fold = F(result.x);
    while (1) 
    {
      penalty = gamma*penalty;
      result = meth(internal1, result.x, stopit);
      fnew = F(result.x);
      if (abs((fnew-fold)/fnew) <= 1e-2) { break; }
      fold = fnew;
    }
    // check the solution 
    if ( max(abs(H(result.x)[:])) > 1e-3) {
       printf("maximize_eq: please check the feasibility of the solution\n");
    }  
    // return the solution x
    return result.x;
};


maximize_le = function( F, G, x0, tol, method, strategy)
{
    global (mdsmax, nmsmax)
    // copy the function into internal variable
    funF  = F;
    funG  = G;
    // some parameters used in mdsmax/nmsmax
    if(exist(tol)) {
      stopit = [tol,1e5,inf(),0,0];
    else
      stopit = [1e-5,1e5,inf(),0,0];
    }
    // you can use either mdsmax or nmsmax here
    meth = mdsmax;
    if (exist(method)) {
       if (method==0) {
          meth = mdsmax;
       else
          meth = nmsmax;
       }
    }
    // strategy
    internal2 = exterior2;
    penalty   = 1;  
    gamma     = 3;

    if (exist(strategy)) 
    {
       if (strategy==1){
          internal2 = interior2;
       else if (strategy==2) {
          internal2 = einterior2;
       else if (strategy==3) {
          internal2 = alm2;
       }}} 
    }

    result = meth(internal2, x0, stopit);
    fold = F(result.x);
    while (1) 
    {
      penalty  = gamma*penalty;
      result = meth(internal2, result.x, stopit);
      fnew = F(result.x);
      if (abs((fnew-fold)/fnew) <= 1e-4) { break; }
      fold = fnew;
    }
    // check the solution 
    if ( max(G(result.x)[:]) > 1e-3) {
       printf("maximize_le: please check the feasibility of the solution\n");
    }  
    // return the solution x
    return result.x;
};

maximize_gen = function( F, G, H, x0, tol, method, strategy)
{
    global (mdsmax, nmsmax)
    // copy the function into internal variable
    funF  = F;
    funG  = G;
    funH  = H;
    // some parameters used in mdsmax/nmsmax
    if(exist(tol)) {
      stopit = [tol,1e5,inf(),0,0];
    else
      stopit = [1e-5,1e5,inf(),0,0];
    }
    // you can use either mdsmax or nmsmax here
    meth = mdsmax;
    if (exist(method)) {
       if (method==0) {
          meth = mdsmax;
       else
          meth = nmsmax;
       }
    }
    // strategy
    internal3 = exterior3;
    penalty   = 1;  
    gamma     = 3;

    if (exist(strategy)) 
    {
       if (strategy==1){
          internal3 = interior3;
       else if (strategy==2) {
          internal3 = einterior3;
       else if (strategy==3) {
          internal3 = alm3;
       }}} 
    }

    result = meth(internal3, x0, stopit);
    fold = F(result.x);
    while (1) 
    {
      penalty  = gamma*penalty;
      result = meth(internal3, result.x, stopit);
      fnew = F(result.x);
      if (abs((fnew-fold)/fnew) <= 1e-4) { break; }
      fold = fnew;
    }
    // check the solution 
    if ( max(G(result.x)[:]) > 1e-3) {
       printf("maximize_gen: please check the feasibility of the solution\n");
    }  
    // return the solution x
    return result.x;
};

exterior1 = function(x)
{
    // exterior penalty function method, eq constraints
    return funF(x)-penalty*sum((funH(x).^2)[:]);
};

exterior2 = function(x)
{
    // exterior penalty function method, le constraints
    gv = funG(x)[:];
    if (all(gv <= 0)) {
       v = funF(x);
    else
       v = funF(x) - penalty*sum(gv[find(gv>0)].^2);
    }
    return v;
};

exterior3 = function(x)
{
    // exterior penalty function method, gen constraints
    gv = funG(x)[:];
    if (all(gv <= 0)) {
       v = funF(x) - penalty*sum((funH(x).^2)[:]);
    else
       v = funF(x) - penalty*(sum((funH(x).^2)[:])+sum(gv[find(gv>0)].^2));
    }
    return v;
};
